package Package1;


public class CbrStats {

    static int decodePc(int currentPitch) {
        return ((currentPitch % 1000) / 10);
    }

    static int decodeOctave(int currentPitch) {
        return currentPitch / 1000;
    }

    public static void main(String[] args) {

        if (args.length < 1) {
            System.out.println("E' necessario immettere una serie di valori espressi in codifica CBR");
            System.exit(1);
        }

        int[] cbrSequence = new int[args.length];
        try {
            for (int i = 0; i < args.length; i++) {
                cbrSequence[i] = Integer.parseInt(args[i]);
                // verifica sulla correttezza della codifica cbr in ingresso:
                //   nc                            pc
                if ((cbrSequence[i] % 10 > 6) || ((cbrSequence[i] % 1000) / 10) > 11) {
                    System.out.println("Valore immesso non valido nella codifica cbr: " + cbrSequence[i]);
                    System.exit(1);
                }
            }
        } catch (NumberFormatException e) {
            System.out.println("Gli argomenti devono essere di tipo intero.");
            System.exit(1);
        }

        double maxF = Float.MIN_VALUE;
        double minF = Float.MAX_VALUE;
        double sum = 0;

        double halftoneRatio = Math.pow(2, (double) 1 / 12); // radice dodicesima di due

        for (int currentPitch : cbrSequence) {
            // calcolo lo spiazzamento in semitoni dal La dell'ottava centrale (PC=9), cui corrisponde f = 440Hz
            int halftonesOffset = decodeOctave(currentPitch) * 12 + decodePc(currentPitch) - (12 * 4 + 9);
            double currentF = 440 * Math.pow(halftoneRatio, halftonesOffset);
            if (currentF > maxF)
                maxF = currentF;
            if (currentF < minF)
                minF = currentF;
            sum += currentF;
        }
        double averageF = sum / cbrSequence.length;
        System.out.println("Frequenza del pitch piu' acuto: " + String.format("%.2f", maxF) + "Hz"); // per formattare il numero con 2 soli decimali
        System.out.println("Frequenza del pitch piu' grave: " + String.format("%.2f", minF) + "Hz");
        System.out.println("Media aritmetica delle frequenze: " + String.format("%.2f", averageF) + "Hz");
    }

}









